# -*- coding:utf-8 -*-
"""
Danpen - code snippet management software
===================================================================

Licence (MIT)
-------------
 
Copyright (c) 2012-2013, Shin Fujihara.
 
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os
import codecs
import time, math
import datetime

from bottle import route, run, view, static_file, url, request, redirect

NUM62 = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
LANG_LIST = ('C#', 'Java', 'JavaScript', 'Perl', 'PHP', 'Python', 'Ruby', 'VB', 'VBScript')
DATA_DIR =  os.path.join(os.path.dirname(__file__), "var")


@route("/favicon.ico")
def favicon():
    return static_file("favicon.ico", root="./statics")

@route('/statics/<filepath:path>', name='statics')
def statics(filepath):
    return static_file(filepath, root="./statics")
    
    
@route("/", name="index")
@view("index")
def index():
    """トップページの表示"""
    return dict(url=url)

@route("/", method="POST")
def create():
    """Danpenの保存"""
    
    #保存用のディレクトリ作成
    path = DATA_DIR
    if not os.path.exists(path):
        os.makedirs(path)
    
    #保存するファイル名を作成
    filename = create_filename( request.forms.get("lang-select") )
    while os.path.exists(os.path.join(path, filename)):
        filename = create_filename()
    
    f = codecs.open(os.path.join(path, filename), 'w', 'utf-8')
    if get_parameter("description") == "":
        f.write("description\n")
    else:
        f.write(get_parameter("description") + "\n")
    f.write(get_parameter("lang-document"))
    f.close()
    
    redirect(url('show', doc_id = filename))


@route("/update/<doc_id>", method="POST", name="update")
def update(doc_id):
    """Danpenの更新"""
    path = DATA_DIR
    f = codecs.open(os.path.join(path, doc_id), 'w', 'utf-8')
    
    if get_parameter("description") == "":
        f.write("description\n")
    else:
        f.write(get_parameter("description") + "\n")
    f.write(get_parameter("lang-document"))
    f.close()
    
    redirect(url('show', doc_id = doc_id))


@route("/remove/<doc_id>", method="POST", name="remove")
def remove(doc_id):
    """Danpenの削除"""
    path = DATA_DIR
    file_path = os.path.join(path, doc_id)
    
    if os.path.exists(file_path):
        os.remove(file_path)
    
    redirect(url('index'))


@route("/show/<doc_id>", name="show")
@view("show")
def show(doc_id):
    """ 指定されたDanpenの詳細を表示 """
    path = DATA_DIR
    f = codecs.open(os.path.join(path, doc_id), 'r', 'utf-8')
    lines = f.readlines()
    f.close()
    
    c = {}
    c['url'] = url
    c['doc_id'] = doc_id
    c['lang_name'] = get_lang_name(doc_id)
    
    if 0 < len(lines):
        c['description'] = lines[0]
        c['lang_document'] = "".join(lines[1:])
    else:
        c['description'] = "description"
        c['lang_document'] = ""
    
    return c
    




@route("/all", name="all")
@view("all")
def all():
    """ 登録されているDanpenの一覧を表示 """
    
    
    file_list = []
    
    path = DATA_DIR
    
    if not os.path.exists(path):
        os.makedirs(path)
    
    files = os.listdir(path)
    
    for file in files:
        file_info = {}
        
        file_info['doc_id'] = file
        file_path = os.path.join(path, file)
        
        f = codecs.open(file_path, 'r', 'utf-8')
        file_info['desc'] = f.readline().strip()
        f.close()
        
        if file_info['desc'] == "":
            file_info['desc'] = "description"
        
        file_info['update_time'] = time.strftime('%Y/%m/%d %X',(time.localtime(os.path.getmtime(file_path))))
        
        file_info['lang'] = get_lang_name(file)
        
        file_list.append(file_info)
    
    
    file_list = sorted(file_list, key=lambda l:l['update_time'], reverse=True)
    
    for info in file_list:
        print(info['update_time'], info['lang'])
    
    
    c = {}
    c['url'] = url
    c['file_list'] = file_list
    
    return c



def get_parameter(key):
    return unicode(request.forms.get(key), 'utf-8')


def create_filename(lang):
    """ 保存するファイル名の作成 """
    
    #現在の年月日時間を62進数に変換
    keys = str(time.time()).split('.')

    key = keys[0]
    if len(keys[1]) == 1:
        key += '0'
    key += keys[1]

    v = int(key)
    r = ""
    while (v > 0):
        r = NUM62[int(v % 62)] + str(r);
        v = math.floor(v / 62)
    
    #最後に言語の種類をつけてファイル名にする
    if lang in LANG_LIST:
        r += NUM62[LANG_LIST.index(lang)]
    else:
        r += 'Z'
        
    return r


def get_lang_name(filename):
    """ ファイル名の末尾から、書かれた言語を判定する """
    key = filename[-1]
    index = -1
    if key in NUM62:
        index = NUM62.index(key)
        
    if 0 <= index < len(LANG_LIST):
        return LANG_LIST[index]
    
    return 'UnKnown'
    

if __name__ == '__main__':
    run(host='0.0.0.0', port=8080, debug=True, reloader=True)






